// Carl Nygaard

// This file describes the "class" pucChecker.

// This class abstracts out an arbitrary checker that 
// can be created by the user (aka checker developer).
// The user's code and bank are stored for them in XML

function pucChecker() {

  // name is kept unique within a configuration because it is the ID
  this.name = "";
  this.description = "";
	
  this.code = "";
 
  this.defaultBank = new pucBank();
  this.defaultBank.name = "default";
  
  // Methods that can be overloaded by the user:
  
  // Default simple comparrison with no fluff, to be overloaded by developer
  this.checkForUpdates = function(page) { 
    puc_logMessage("Warning: Current checker is not overloading the main checking function");
    return (page.newSource != page.oldSource);
  }

  // Returning null will result in not adding a value
  this.onUserCreateNew = function(bank, key) {
    return prompt("Please enter a new value","");
  }

  // Returning null will result in not adding a value
  this.onUserEdit = function(bank, key, valueIndex) {
    return prompt("Please enter a new value", bank.getValuesFromKey(key)[valueIndex]);
  }

  // Returning true will allow the page be remove,
  // returning false will not
  this.onUserRemove = function(bank, key, valueIndex) {
    return confirm("Are you sure you want to delete this " + key.toLowerCase() + "?");
  }

  this.showDiff = function(page, oldSource, newSource) {
    return "Difference not available with the current Checker";
  }


  // These methods should not be overloaded:

  this.copyInitialize = function(other) {
    with(this) {
      name = other.name;
      description = other.description;
      code = other.code;
      defaultBank = new pucBank();
      defaultBank.copyInitialize(other.defaultBank);
    }

    this.loadCode();  // Execute developer code

    return this;
  }

  // Sample XML:
  // <checker name="Filter" description="Filters things a lot"> 
  //   <bank>...(this is the "defaultBank") ..</bank>
  //   <script><![CDATA[
  //     var theAnswer = 42;
  //   ]]></script>
  // </checker>

  // Initializes this object from an XML node
  this.XMLInitialize = function(node) {
    this.name = node.getAttributeNode("name").value;
    this.description = node.getAttributeNode("description").value;

    var subNodes = 0;
    var subNode = null;
    for(; (subNode = node.childNodes[subNodes]); subNodes++) {
      if(subNode.nodeName == "bank") {
        this.defaultBank.XMLInitialize(subNode);
      } else if(subNode.nodeName == "script") {
        var scriptNodes = 0;
        var scriptNode = null;
        this.code = "";
        for(; (scriptNode = subNode.childNodes[scriptNodes]); scriptNodes++) {
          this.code += scriptNode.nodeValue; 
        }
      }
    }

    this.loadCode();    // Execution of developer code
    return this;
  }

  this.toXMLNode = function(doc) {
    var checkerNode = doc.createElement("checker");
    checkerNode.setAttribute("name", this.name);
    checkerNode.setAttribute("description", this.description);

    checkerNode.appendChild(doc.createTextNode("\n\t\t"));
    checkerNode.appendChild(this.defaultBank.toXMLNode(doc));

    checkerNode.appendChild(doc.createTextNode("\n\t\t"));

    var codeNode = doc.createElement("script");
    codeNode.appendChild(doc.createCDATASection(this.code));

    checkerNode.appendChild(codeNode);

    checkerNode.appendChild(doc.createTextNode("\n\t"));

    return checkerNode;
  }

  this.loadCode = function() {
    var lineNumber;
    try {
      // Get eval()'s line number so that a useful number will be produced in a 
      // user error message
      try { null.generateError(); } catch(ex) { lineNumber = ex.lineNumber+1;}
      eval(this.code);
    } catch(e) {
      puc_logMessage("Error while trying to execute developer code for Checker " + this.name + 
                     " on Line " + (e.lineNumber - lineNumber) + ":\n" + e);
    }
  }
}