const PUC_GUID = "{48998E1D-262B-47cb-9D9F-02D1906F1561}";
const PUC_VERSION = "0.3";

// File manipulation functions:
function puc_getReadStream (fileName) {
        var file =  puc_getFile(fileName);
        return puc_getStream(file);
}

// Opens a file from the chrome directory
function puc_getReadStreamChromeDir (fileName) {
        var file =  puc_getChromeFile(fileName);
        return puc_getStream(file);
}


function puc_getStream(theFile) {
        var stream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
        stream.init(theFile, 0x01 , 00004, null);
        var sInputStream = Components.classes["@mozilla.org/network/buffered-input-stream;1"].createInstance(Components.interfaces.nsIBufferedInputStream);
        sInputStream.init(stream, 1000);
        return sInputStream;
}


function puc_getWriteStream(fileName) {
        var file = puc_getFile(fileName);
        return puc_getWriteStreamWithFile(file);
}

function puc_getWriteStreamChromeDir(fileName) {
        var file = puc_getChromeFile(fileName);
        return puc_getWriteStreamWithFile(file);
}

function puc_getWriteStreamWithFile(file) {
        var stream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
        stream.init(file, 0x02 | 0x08 | 0x20, 420, 0);
        return stream;
}

// puc_toUnicode and puc_fromUnicode taken slightly modified from the
// Quicknotes extension
function puc_toUnicode(str, charset) {
// Removed error handling so that a conversion error
// will default to the old (direct) method of reading the data
/*  try{
*/	
    var uniConv = Components.classes['@mozilla.org/intl/scriptableunicodeconverter']
                    .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    uniConv.charset = charset;
    var data = uniConv.ConvertToUnicode(str);
/*  } catch(e) {
    puc_logMessage("Error converting to Unicode:\n"+e);
  }
*/
  return data;
}

function puc_fromUnicode(str, charset) {
  try {
    var uniConv = Components.classes['@mozilla.org/intl/scriptableunicodeconverter']
                   .createInstance(Components.interfaces.nsIScriptableUnicodeConverter);
    uniConv.charset = charset;
    var data = uniConv.ConvertFromUnicode(str);
    data += uniConv.Finish();
  }
  catch(e) {
    puc_logMessage("Error converting from Unicode:\n"+e);
  }
  return data;
}

function puc_getFile(fileName) {
  var file = puc_getDir();
        file.append(fileName);
        return file;
}

function puc_getChromeFile(fileName) {
  var file = puc_getChromeDir();
        file.append(fileName);
        return file;
}


function puc_getDir() {
        var type = "ProfD";
        var file = Components.classes["@mozilla.org/file/directory_service;1"]
                                .getService(Components.interfaces.nsIProperties)
                                .get(type, Components.interfaces.nsILocalFile);
        
        file.append("extensions");
        file.append(PUC_GUID);
        file.append("store");

        return file;
}

function puc_getChromeDir() {
        var type = "UChrm";
        var file = Components.classes["@mozilla.org/file/directory_service;1"]
                                .getService(Components.interfaces.nsIProperties)
                                .get(type, Components.interfaces.nsILocalFile);
        return file;
}


function  pucDoOptions()
{
    window.openDialog("chrome://puc/content/pucOptions.xul", "pucOptions", "centerscreen,modal");
}


function puc_logMessage(logString)
{
  if(PrefUtils.getPref("puc.showactivity")) {
    aConsoleService = Components.classes["@mozilla.org/consoleservice;1"].getService(Components.interfaces.nsIConsoleService);

    if(aConsoleService)
      aConsoleService.logStringMessage("PUC: " + logString);
  }
}

// Returns the window that is currently managing PUC, (it should return null if there are no browsers)
function pucGetMasterWindow() 
{
  var pucMasterWindow = null;
  // Find the browser window holding the configuration info
  var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
  var enumerator = wm.getEnumerator("navigator:browser");
  while(enumerator.hasMoreElements()) {
    pucMasterWindow = enumerator.getNext();
    if(pucMasterWindow.pucIWatchPages) 
      break;
  }
  return pucMasterWindow;
}

// The next two functions were taken from http://developer.mozilla.org/en/docs/Using_nsIXULAppInfo on 12/31/05

// Throws an error in firefox versions before 1.5
function puc_isVersionAtLeast(strVersion) {
  // assuming we're running under Firefox
  var appInfo = Components.classes["@mozilla.org/xre/app-info;1"]
                          .getService(Components.interfaces.nsIXULAppInfo);
  var versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
                                .getService(Components.interfaces.nsIVersionComparator);
  //puc_logMessage("Firefox Version: " + appInfo.version);                                
  return versionChecker.compare(appInfo.version, strVersion) >= 0;
}

function getAppID() {
  var id;
  if("@mozilla.org/xre/app-info;1" in Components.classes) {
    // running under Mozilla 1.8 or later
    id = Components.classes["@mozilla.org/xre/app-info;1"]
                   .getService(Components.interfaces.nsIXULAppInfo).ID;
  } else {
    try {
      id = Components.classes["@mozilla.org/preferences-service;1"]
                     .getService(Components.interfaces.nsIPrefBranch)
                     .getCharPref("app.id");
    } catch(e) {
      // very old version
      puc_logMessage("Error reading Firefox ID: " + e);
    }
  }
  return id;
}

function puc_isAtLeastFirefoxV1_5()
{
  try {
    // This throws an error in versions of firefox before 1.5
    if(puc_isVersionAtLeast("1.5"))
      return true;
    else
      return false;
  } catch(e) {
    return false;
  }
  
}

//puc_logMessage("It is " + puc_isAtLeastFirefoxV1_5() + " that you are using firefox 1.5 or higher");