// This is used within the options for the development tab

// The name of the selected checker, which must be unique.
// We use names for uniqueness because IDs wouldn't transfer easily between people's machines
var currentChecker = null;

function pucDevelopmentLoad()
{
  // We rely on pucLoad to have already opened the configuration to tempConfig
  pucLoadCheckersIntoWindow();
}


function pucLoadCheckersIntoWindow()
{
  var listCheckers =  document.getElementById("listCheckers");

  // First remove any items we might already have
  while(listCheckers.hasChildNodes())
    listCheckers.removeChild(listCheckers.childNodes[0]);
  currentChecker = null;
  for (var i = 0; i < tempConfig.checkers.length; i++) {
    if ( tempConfig.checkers[i] != null ) {
      var newItem = listCheckers.appendItem(tempConfig.checkers[i].name, tempConfig.checkers[i].name);
      newItem.setAttribute("tooltiptext", tempConfig.checkers[i].description);
    }
  }
}

function doCheckerSelect()
{
  assureCheckerSaved();

  var list = document.getElementById("listCheckers");
  if (list.selectedItem != null)
  {
     var selectedChecker = list.selectedItem.value;
     if (selectedChecker != null)
       doLoadCheckerDetails(selectedChecker);
  }
}

function assureCheckerSaved() {
  // Save any changes that weren't changed in the last page
  // This is meant to catch the case when the user changes a value
  // withiout losing focus and selecting another page.
  if(currentChecker != null) {
    var pucName =  document.getElementById("checker-name");
    var pucDesc =  document.getElementById("checker-description");
    var pucCode =  document.getElementById("checker-code");

    // The explicit checks are to prevent always
    // igniting the handlers for these elements
    with(currentChecker) {
      if(pucName.value != name)
        onCheckerNameChanged();

      if(pucDesc.value != description) 
        onCheckerDescriptionChanged();

      if(pucCode.value != code)
        onCheckerCodeChanged();
     }        
  }
}


function doLoadCheckerDetails(name)
{
  var pucName =  document.getElementById("checker-name");
  var pucDesc =  document.getElementById("checker-description");
  var pucCode =  document.getElementById("checker-code");

  currentChecker = tempConfig.getCheckerFromName(name);

  pucName.value = currentChecker.name;
  pucDesc.value = currentChecker.description;
  pucCode.value = currentChecker.code;
}


function doNewChecker() {
  var newChecker = new pucChecker();
  newChecker.name = "Untitled";

  currentChecker = tempConfig.addChecker(newChecker);
  pucLoadCheckersIntoWindow();
  pucSetDirtyFlag();
}


function doDuplicateChecker() {
  var newChecker = new pucChecker();
  if(currentChecker != null) {
    newChecker.copyInitialize(currentChecker);

    // addChecker will assure the name is unique for us
    tempConfig.addChecker(newChecker);
    pucLoadCheckersIntoWindow();
    pucSetDirtyFlag();
  }
}

function doRemoveChecker() {
  if(currentChecker != null) {
    if(confirm(strings.getString("confirm.remove.checker"))) {
      tempConfig.removeChecker(currentChecker);
      pucLoadCheckersIntoWindow();
      pucSetDirtyFlag();
    }
  }
}

function onCheckerNameChanged()
{
  if(currentChecker != null && confirm(strings.getString("confirm.change.checker.name"))) {
    var checkername =  document.getElementById("checker-name");

    currentChecker.name = checkername.value;
    tempConfig.makeCheckerNameUnique(currentChecker);

    // If the name changes we need to update the name in the current list of checkers
    var list = document.getElementById("listCheckers");
    if (list.selectedItem != null)
      list.selectedItem.label = checkername.value;
  
    pucSetDirtyFlag();
  }
}

function onCheckerDescriptionChanged()
{
  if(currentChecker != null) {
    var checkerdesc =  document.getElementById("checker-description");
  
    currentChecker.description = checkerdesc.value;
  
    // If the desc changes we need to update the tooltip
    var list = document.getElementById("listCheckers");
    if (list.selectedItem != null)
      list.selectedItem.setAttribute("tooltiptext", checkerdesc.value)
  
    pucSetDirtyFlag();
  }
}

function onCheckerCodeChanged()
{
  if(currentChecker != null) {
    var checkercode =  document.getElementById("checker-code");
    currentChecker.code = checkercode.value;
    pucSetDirtyFlag();
  }
}

function doSetDefaultBank() {
  currentChecker.defaultBank.doUserEdit(currentChecker, true);
  pucSetDirtyFlag();
}

function doExportPucChecker() {
  var doc = document.implementation.createDocument("", "pucconfig", null);

  if(currentChecker) {
    var checkerNode = currentChecker.toXMLNode(doc);
    doc.firstChild.appendChild(doc.createTextNode("\n\t"));
    doc.firstChild.appendChild(checkerNode);
    doc.firstChild.appendChild(doc.createTextNode("\n"));
    doc.firstChild.setAttribute("version", PUC_VERSION);
    doExportPuc(doc);
  }
}